class Intercepted
  def foo(bar)
    if block_given?
      yield(bar)
    else
      bar
    end
  end

  def self.foo(bar)
    if block_given?
      yield(bar)
    else
      bar
    end
  end
end

describe Intercepted do
  describe '#intercept' do
    it 'intercepts a class method call and yields the method object, the args and the block' do
      strawman = double('strawman')
      expect(strawman).to receive(:hit!)

      intercept(Intercepted, :foo) do |method, *args, &block|
        strawman.hit!
        expect(args).to eq(['foo'])
        expect(block.call('bar')).to eq('baz')
        expect(method.call('bar', &block)).to eq('baz')
        'blah'
      end

      expect(
        Intercepted.foo('foo') do |bar|
          expect(bar).to eq('bar')
          'baz'
        end
      ).to eq('blah')
    end
  end

  describe '#intercept_any_instance_of' do
    it 'intercepts an instance method call and yields the method object, the args and the block' do
      strawman = double('strawman')
      expect(strawman).to receive(:hit!)

      intercept_any_instance_of(Intercepted, :foo) do |method, *args, &block|
        strawman.hit!
        expect(args).to eq(['foo'])
        expect(block.call('bar')).to eq('baz')
        expect(method.call('bar', &block)).to eq('baz')
        'blah'
      end

      expect(
        Intercepted.new.foo('foo') do |bar|
          expect(bar).to eq('bar')
          'baz'
        end
      ).to eq('blah')
    end
  end
end

